#' @title Adapt sirene data
#'
#' @description 
#' The following treatment are performed:
#' - Only some fileds are selected, field names (colnames) are consistently set.
#' - NA values are consistently set.
#' - Type of fields are consistently set, some checks are performed on transformations.
#' - Data set is cut and saved by departments
#' - Only observations with spatial coordinates are kept.
#' - Exposition of observations (belonging or not to eaip) is added.
#'
#' @param origin character, path to the directory where archive are stored
#' @param destination character, path to the directory where results should be saved
#' @param archive character, vector of archive to be adpated
#' @param export character, option for export format of the results
#' @param scheme data.frame, how archive variables should be treated
#' @param na_value data.frame, list of NA values for each variables
#' @param eaip_path character, path where eaip archives should be find
#' @param projection optional numeric, 4 digits projection to use
#' @param order_by optional character, name of the variable to sort the output data. 
#' Default sorting is by state and eaip (if added).
#' @param retrieve logical, should the result be returned
#' @param verbose logical, should the function give some sumup informations.
#'
#' @return if retrieve = TRUE, data.frame of adapated data stored in archive.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Victor Champonnois et Frédéric Grelot
#' 
#' @examples
#' 
#' \dontrun{
#' origin = ""
#' destination = tempdir()
#' archive = "geo_siret_34.csv.gz"
#' result = adapt.geo_sirene(origin, destination, verbose = TRUE, retrieve = TRUE)
#' }

adapt.geo_sirene = function(origin,
    destination,
    archive,
    export = "fst",
    scheme = floodam.data::scheme_sirene_2019,
    na_value = floodam.data::scheme_sirene_na,
    eaip_path = NULL,
    projection = 4326,
    order_by = c("state", "eaip"),
    retrieve = FALSE,
    verbose = FALSE) {

    ###  Recursive call if archive is missing
    if (missing(archive)) {archive = list.files(origin, pattern = ".csv.gz$")}
    if (length(archive) > 1) 
        return(lapply(archive,
            adapt.geo_sirene,
            origin = origin,
            destination = destination,
            export = export,
            scheme = scheme,
            na_value = na_value,
            eaip_path = eaip_path,
            projection = projection,
            order_by = order_by,
            retrieve = retrieve,
            verbose = verbose))
    if (length(archive) == 0)
        return(NULL)

    ###  Analyse archive
    path = file.path(origin, archive)
    analyse = regmatches(archive, regexec("(.*?)[.](csv[.]gz|gz)$", archive, perl = TRUE))[[1]]
    if (length(analyse) == 0) return(NULL)
    name = analyse[2]
    extension = analyse[3]
    location = utils::tail(strsplit(origin, "/")[[1]], 1)

    ### Start journal information
    dir.create(destination, showWarnings = FALSE, recursive = TRUE) # Check if error may be generated.
    start_time = Sys.time()
    journal = add_journal_old(
        "WARNING: This file is automatically produced by floodam.data. Do not edit by hand.", "",
        "Contents", "\t1. General log informations", "\t2. Stats on removed data", "\t3. Stats on exposition", "",
        first = TRUE)
    journal = add_journal_old(
        sprintf("%s from '%s' adpated by floodam.data (%s)", archive, location, utils::packageVersion("floodam.data")),
        journal = journal, verbose = verbose, first = TRUE)
    journal = add_journal_old(sprintf("Starting time: %s", Sys.time()), journal = journal, verbose = verbose)
    on.exit(journal <- add_journal_old("", 
        sprintf("Total time elapsed for %s from '%s': %s", name, location, fmt(Sys.time() - start_time)),
        journal = journal, verbose = verbose, first = TRUE))
    on.exit(write(journal, file.path(destination, sprintf("%s.%s", name, "log"))), add = TRUE)
    
    ### Read file and do selection, change colnames, start easy formatting
    scheme = scheme[order(scheme[["order"]]), ]
    colClasses = ifelse(scheme[["keep"]], "character", "NULL")
    colClasses[scheme[["type"]] == "integer" & scheme[["keep"]]] = "integer"
    colClasses[scheme[["type"]] == "numeric" & scheme[["keep"]]] = "numeric"
    scheme = scheme[scheme[["keep"]], ]
    rownames(scheme) = scheme[["name"]]

    size = c(inital = file.info(path)[["size"]])
    result = data.table::fread(path, colClasses = colClasses, na.strings = "", 
        data.table = FALSE, col.names = scheme[["name"]])

    ### Mores adaptations
    # NA treament
    for (i in 1:nrow(na_value)) {
        var = na_value[i, "name"]
        result[[var]] = treat_na(result[[var]], na_value[i, "na.value"])
    }

    # format factors (after NA treatment)
    selection = rownames(scheme)[scheme[["type"]] == "factor"]
    result[selection] = lapply(result[selection], as.factor)

    # format Date (after NA treatment)
    selection = rownames(scheme)[scheme[["type"]] == "Date"]
    result[selection] = lapply(result[selection], as.Date)
    for (var in selection) {
        if (all(is.na(result[[var]]))) {
            journal = add_journal_old(sprintf("Possible format error for '%s'. All values are NA.", var),
                journal = journal, verbose = verbose)
        }
    }

    # format logical
    result[["employer"]] = to_logical(result[["employer"]], "O", "N")
    result[["hq"]] = to_logical(result[["hq"]], "true", "false")

    selection = rownames(scheme)[scheme[["type"]] == "logical"]
    for (var in selection) {
        if (all(is.na(result[[var]]))) {
            journal = add_journal_old(sprintf("Possible format error for '%s'. All values are NA.", var),
                journal = journal, verbose = verbose)
        }
    }
    
    # Check type
    test = sapply(result, class) == scheme[["type"]]
    if (all(test)) {
        journal = add_journal_old("All variables have the good type.", journal = journal, verbose = verbose)
    } else {
        journal = add_journal_old(
            sprintf("Some variables do not have the good type. Check %s.",
                paste("'", names(test)[!test], "'", sep = "", collapse = ", ")),
            journal = journal, verbose = verbose)
    }
    
    int_time = Sys.time()
    journal = add_journal_old(
        sprintf("Time elapsed: %s", fmt(int_time - start_time)),
        journal = journal, verbose = verbose)

    ### GIS adaptations by departments

    result = split(result, substring(result[["commune"]], 1, 2))
    
    # Remove some departements ?
    departement_remove = grep("98", names(result))
    if (length(departement_remove) > 0) result = result[-departement_remove]

    stat_remove = list()
    stat_eaip = list()

    for (d in names(result)) {
        journal = add_journal_old("",
            sprintf("Treating department %s in %s from '%s'.", d, name, location),
            journal = journal, verbose = verbose, first = TRUE)        
        
        # Treat observations with missing spatial coordinates
        result[[d]][["coord"]] = stats::complete.cases(result[[d]][, c("longitude", "latitude")])
 
        stat = table(result[[d]][c("state", "coord")])
        colnames(stat) = c("missing", "present")

        journal = add_journal_old(
            sprintf("Observations removed due to missing spatial coordinates: %s over %s, %s remaining (%1.1f%%)",
                sum(stat[, "missing"]), sum(stat), sum(stat[, "present"]), 
                sum(stat[, "present"]) / sum(stat) * 100),
            journal = journal, verbose = verbose)
        stat_remove[[d]] = as.data.frame(stat)

        # Removing observations with missing spatial coordinates
        result[[d]] = result[[d]][result[[d]][["coord"]],]
        result[[d]][["coord"]] = NULL        

        # Transform in sf format
        result[[d]] = sf::st_as_sf(result[[d]], coords =c("longitude", "latitude"), crs = projection)

        # Adding information from eaip
        if (!is.null(eaip_path)) {
            if (is.character(eaip_path)) {
                eaip = try(
                    suppressWarnings(readRDS(file.path(eaip_path, sprintf("eaip_%s.rds", format_scope(d))))),
                    silent = TRUE
                )
            }
            if ("try-error" %in% class(eaip)) {
                journal = add_journal_old(sprintf("'eaip' analysis not included: '%s' is not a path to an rds archive.", path), 
                    journal = journal,
                    verbose = verbose)
            } else if (!"sf" %in% class(eaip)) {
                journal = add_journal_old(
                    sprintf("'eaip' analysis not included: '%s' is not a sf object.", path),
                    journal = journal,
                    verbose = verbose)
            } else {
                eaip = sf::st_transform(eaip, crs = projection)
                result[[d]][["eaip"]] = in_layer(result[[d]], eaip)

                stat = table(unclass(result[[d]])[c("state", "eaip")])
                colnames(stat) = c("out", "in")

                journal = add_journal_old(
                    sprintf("'eaip' analysis included: %s in eaip over %s (%1.1f%%)",  
                        sum(stat[, "in"]), sum(stat), sum(stat[, "in"]) / sum(stat) * 100),
                    journal = journal, verbose = verbose)
                stat_eaip[[d]] = as.data.frame(stat)
            }
        }

        ### Ordering
        selection = order_by[order_by %in% names(result[[d]])]
        if (length(selection) == 0) {
            journal = add_journal_old(
                "Result unsorted. 'order_by' does not contain any admissible name for geo_sirene data.",
                journal = journal, verbose = verbose)
        } else {
            result[[d]] = result[[d]][do.call(order, unclass(result[[d]])[selection]), ]
            journal = add_journal_old(
                sprintf("Result sorted by %s", paste("'", selection, "'", sep = "", collapse = " + ")),
                journal = journal, verbose = verbose)
        }

        ### Save result
        if ("rds" %in% export) {
            if (length(result) == 1) {
                path = file.path(destination, sprintf("%s.%s", name, "rds"))
            } else {
                path = file.path(destination, sprintf("%s-%s.%s", name, d, "rds"))
            }
            saveRDS(result[[d]], path)
            journal = add_journal_old(
                sprintf("Result saved as %s. Size: %s (%0.3f of inital)", 
                    path, file.info(path)[["size"]], file.info(path)[["size"]] / size),
                journal = journal, verbose = verbose)
        }
        if ("fst" %in% export) {
            sf::st_geometry(result[[d]]) = NULL
            if (length(result) == 1) {
                path = file.path(destination, sprintf("%s.%s", name, "fst"))
            } else {
                path = file.path(destination, sprintf("%s-%s.%s", name, d, "fst"))
            }
            fst::write.fst(result[[d]], path, compress = 100)
            journal = add_journal_old(
                sprintf("Result saved as %s. Size: %s (%0.3f of inital)", 
                    path, file.info(path)[["size"]], file.info(path)[["size"]] / size),
                journal = journal, verbose = verbose)
        }
        if (all(! export %in% c("rds", "fst"))) {
            journal = add_journal_old(
                sprintf("Result not saved: '%s' not admissible as export", export),
                journal = journal, verbose = verbose)
        }


        journal = add_journal_old(
        sprintf("Time elapsed: %s", fmt(Sys.time() - int_time)),
        journal = journal, verbose = verbose)
        int_time = Sys.time()
    }

    ### Add useful journal information
    journal_end = length(journal)
    journal[4] = paste(sep = "",
        journal[4], sprintf(": 8-%s", journal_end + 2)
    )
    
    stat_remove = do.call(rbind, stat_remove)
    if (!is.null(stat_remove)) {
        stat_remove[["departement"]] = substring(rownames(stat_remove), 1, 2)
        stat_remove = stat_remove[c("departement", "state", "coord", "Freq")]
        stat_remove[["Freq"]] = as.integer(stat_remove[["Freq"]])
        row.names(stat_remove) = NULL
        journal[5] = paste(sep = "",
            journal[5], 
            sprintf(": %s-%s", journal_end + 4, journal_end + 4 + nrow(stat_remove))
        )
        journal_end = journal_end + nrow(stat_remove)
        on.exit(suppressWarnings(
            write(c("", utils::capture.output(stat_remove)), file.path(destination, sprintf("%s.%s", name, "log")),
                append = TRUE)),
            add = TRUE)
    } else {
        journal[5] = paste(sep = "", journal[5], ": none")
    }

    stat_eaip = do.call(rbind, stat_eaip)
    if (!is.null(stat_eaip)) {
        stat_eaip[["departement"]] = substring(rownames(stat_eaip), 1, 2)
        stat_eaip = stat_eaip[c("departement", "state", "eaip", "Freq")]
        stat_eaip[["Freq"]] = as.integer(stat_eaip[["Freq"]])
        row.names(stat_eaip) = NULL
        journal[6] = paste(sep = "",
            journal[6], 
            sprintf(": %s-%s", journal_end + 6, journal_end + 6 + nrow(stat_eaip))
        )
        on.exit(suppressWarnings(
            write(c("", utils::capture.output(stat_eaip)), file.path(destination, sprintf("%s.%s", name, "log")),
                append = TRUE)),
            add = TRUE)
    } else {
        journal[6] = paste(sep = "", journal[6], ": none")
    }

    ### Retrieve result
    if (retrieve) {
        if (length(result) == 1) return(invisible(result[[1]]))
        return(invisible(result))
    }
}
