#' @title Adapt gaspar database
#' 
#' @description
#' The following treatments are performed:
#' - data is adapated consistently with theme
#' - only data corresponding to flood hazards are kept
#' - if scope is given, a subpart of data is also saved
#'
#' @param origin character, path to the directory where archive are stored.
#' @param destination character, path to the directory where results should be saved
#' @param archive character, vector of archive to be adpated
#' @param theme character, name of the theme to be treated
#' @param version character, version of data to be treated. If missing, version
#'      is guessed from folders in origin.
#' @param scope list, named list of scope (vector of commune)
#' @param export character, option for export format of the results
#' @param retrieve logical, should the result be returned
#' @param keep logical, should the original data be kept. Default to FALSE.
#' @param verbose logical, should the function give some sumup informations. Default to TRUE.
#'
#' @return if retrieve = TRUE, data.frame of adapated data stored in archive.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot
#' 
#' @examples
#' 
#' \dontrun{
#' origin = "data-local/georisques/gaspar"
#' adapt_gaspar(file.path(origin))
#' adapt_gaspar(file.path(origin, "2022-02-28"), scope = list("so_ii" = so.ii::so_ii_scope))
#' }

adapt_gaspar = function(
    origin,
    destination = origin,
    archive = "gaspar.zip",
    theme = "catnat",
    version,
    scope = NULL,
    export = "rds",
    retrieve = FALSE,
    keep = FALSE,
    verbose = TRUE
) {
    # Prepare log and message
    send_message("adapt_gaspar...", appendLF = FALSE, verbose = verbose)
    on.exit(if (file.exists(destination)) write(journal, file.path(destination, "gaspar.log"), append = TRUE))
    on.exit(
        send_message(
            gsub(sprintf("%s: adapt_gaspar", today), "", journal),
            verbose = verbose
        ),
        add = TRUE
    )
    today = as.character(Sys.Date())
    journal = sprintf("%s: adapt_gaspar theme '%s' is not managed", today, theme)
    
    # Select scheme
    scheme = switch(
        theme,
        "azi" = floodam.data::scheme_gaspar_azi,
        "catnat" = floodam.data::scheme_gaspar_catnat,
        "pprn" = floodam.data::scheme_gaspar_pprn,
        "risq" = floodam.data::scheme_gaspar_risq,
        return(invisible())
    )

    # Select version and archive
    if (missing(version)) {
        version = utils::head(
            sort(
                dir(origin, pattern ="^[0-9]{4}-[0-9]{2}-[0-9]{2}"),
                decreasing = TRUE
            ), 1
        )
        if (length(version) == 0) {
            journal = sprintf("%s: adapt_gaspar no version to treat in %s", today, origin)
            return(invisible())            
        }
    }
    archive  = file.path(origin, version, archive)
    if (!file.exists(archive)) {
        journal = sprintf("%s: adapt_gaspar '%s' does not exist", today, archive)
        return(invisible())
    }

    # Extract information from archive
    utils::unzip(archive, files = sprintf("%s_gaspar.csv", theme), exdir = file.path(origin, version))
    result = utils::read.csv2(
        file.path(origin, version, sprintf("%s_gaspar.csv", theme)),
        colClasses = "character")
    unlink(file.path(origin, version, sprintf("%s_gaspar.csv", theme)))

    # Application of scheme

    ## Temporary pretreatment necessary for catnat theme because of data issues
    if (theme == "catnat") {
        result$num_risque_jo = gsub(" ", "", result$num_risque_jo)
        result$lib_risque_jo = gsub("\uFFFD", "\u00E9", result$lib_risque_jo)
        num_risque_jo = stats::setNames(
            floodam.data::catnat_classification$catnat,
            floodam.data::catnat_classification$catnat_label_fr
        )
        selection = result$num_risque_jo == ""
        result$num_risque_jo[selection] = num_risque_jo[result$lib_risque_jo[selection]]
        if (nrow(unique(result[c("num_risque_jo", "lib_risque_jo")])) !=
                nrow(floodam.data::catnat_classification)) {
            stop("Trouble with catnat_classification. Check compatibility with data")
        }
    }

    result = apply_scheme(result, scheme)

    # Selection of flood hazard
    if (theme == "catnat") {
        hazard = names(floodam.data::catnat_classification)[-(1:2)]
        result = merge(result, floodam.data::catnat_classification)
        result = result[apply(result[hazard], 1, any),]
        result[["catnat"]] = factor(
            as.character(result[["catnat"]]),
            levels = floodam.data::catnat_classification[
                apply(floodam.data::catnat_classification[-(1:2)], 1, any, na.rm = TRUE),
                "catnat"
            ]
        )
    }
    if (theme == "pprn") {
        hazard = names(floodam.data::pprn_classification)[-(1:2)]
        result = merge(result, floodam.data::pprn_classification)
        result = result[apply(result[hazard], 1, any),]

        result["state"] =  analyse_ppr_state(
            result[c(
                "date_prescription",
                "date_deprescription",
                "date_cancellation",
                "date_repeal",
                "date_approval"
            )]
        )
    }

    # Saving
    dir.create(file.path(destination, version), showWarnings = FALSE, recursive = TRUE)
    saveRDS(result, file.path(destination, version, sprintf("%s_gaspar.rds", theme)))
    journal = sprintf(
        "%s: adapt_gaspar '%s' with 'floodam.data' (%s)",
        today,
        file.path(destination, version, sprintf("%s_gaspar.rds", theme)),
        utils::packageVersion("floodam.data")
    )

    # Application of scope
    if (!is.null(scope)) {
        scope = scope[1]
        result = result[result[["commune"]] %in% scope[[1]], ]
        result[["commune"]] = factor(
            as.character(result[["commune"]]),
            levels = scope[[1]]
        )
        saveRDS(result, file.path(destination, version, sprintf("%s_%s.rds", theme, names(scope))))
        journal = sprintf("%s -- scope '%s' treated", journal, names(scope))
    }

    # Remove original data
    if (keep == FALSE) {
        unlink(file.path(origin, version), recursive = TRUE)
        journal = sprintf("%s -- original data removed", journal)
    }

    if (retrieve == TRUE) return(invisible(result)) else invisible(NULL)
}
