#' @title Analyse gaspar to send an alert to mattermost
#' 
#' @description
#' The following treatments are performed:
#' - If no sufficient versions are found, nothing is done.
#' - NA values are consistently set.
#' - Type of fields are consistently set, some checks are performed on transformations.
#' - Data set is cut and saved by departments
#' - Only observations with spatial coordinates are kept.
#' - Exposition of observations (belonging or not to eaip) is added.
#'
# @param origin character, path to the directory where archive are stored
#' @param origin character, path to the directory where archive are stored.
#' @param current character, date of the current data;
#' @param past character, date of the past data.
#' @param scope list, named list of scope (vector of commune).
#' @param alert logical, should the result be returned.
#' @param verbose logical, should the function give some sumup informations.
#'
#' @return if retrieve = TRUE, data.frame of adapated data stored in archive.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot
#' 
#' @examples
#' 
#' \dontrun{
#' origin = "data-local/georisques/gaspar"
#' download_gaspar(file.path(origin, Sys.Date()))
#' catnat = adapt_gaspar(file.path(origin, Sys.Date()), scope = list("so_ii" = so.ii::so_ii_scope))
#' alert_gaspar(origin, scope = "so_ii")
#' }

alert_gaspar = function(origin, current, past, scope, alert = TRUE, verbose = TRUE) {

    send_message("alert_gasper...", appendLF = FALSE, verbose = verbose)
    on.exit(write(journal, file.path(origin, "gaspar.log"), append = TRUE))
    on.exit(
        send_message(
            gsub(sprintf("%s: alert_gaspar", today), "", journal),
            verbose = verbose
        ),
        add = TRUE
    )

    today = as.character(Sys.Date())
    journal = sprintf("%s: alert_gaspar no sufficient version of gaspar in %s", today, origin)

    # theme is set to catnat. To change it, selection of columns mus be adapted...
    theme = "catnat"
    theme_column = c("commune", "catnat_label_fr", "date_start", "date_end")

    available = sort(dir(origin, pattern ="^[0-9]{4}-[0-9]{2}-[0-9]{2}"), decreasing = TRUE)
    if (length(available) < 2) {
        return(invisible())
    }

    if (missing(current)) {
        current = available[1]
    } else {
        if (! current %in% available) {
            journal = sprintf("%s: alert_gaspar %s not in %s", today, current, origin)
            return(invisible())
        }
    }
    if (missing(past)) {
        past = available[2]
    } else {
        if (! past %in% available) {
            journal = sprintf("%s: alert_gaspar %s not in %s", today, past, origin)
            return(invisible())
        }
    }        
    if (missing(scope)) scope = "gaspar"

    archive = sprintf("%s_%s.rds", theme, scope)
    if (! archive %in% dir(file.path(origin, current))) {
            journal = sprintf("%s: alert_gaspar %s not in %s", today, archive, file.path(origin, current))
            return(invisible())        
    }
    if (! archive %in% dir(file.path(origin, past))) {
            journal = sprintf("%s: alert_gaspar %s not in %s.", today, archive, file.path(origin, past))
            return(invisible())       
    }

    data_current = readRDS(file.path(origin, current, archive))
    data_past = readRDS(file.path(origin, past, archive))

    difference = data_current[! data_current[["id"]] %in% data_past[["id"]], theme_column]

    if (nrow(difference) > 0) {
        alert_mattermost(
            c("**New events in gaspar**", "", as.character(knitr::kable(difference, row.names = FALSE))),
            alert = alert
        )
        journal = sprintf("%s: alert_gaspar differences found with %s, message sent to mattermost", today, past)
    } else {
        journal = sprintf("%s: alert_gaspar no differences found with %s", today, past)
    }
    unlink(file.path(origin, past), recursive = TRUE)
}