#' @title Analyse names of archive to get useful infomations.
#'
#' @description
#' The function is set for IGN's names of archive, that should normally be
#' "well" structured with this scheme :
#' data-type_version_precision_format_projection_scope_date.ext. Sometimes a
#' less frequent format is
#' data-name_version_precision_format_projection_scope_proj_date.ext. In the
#' latter case, "proj" is given in another format than "projection", but also in
#' a documented way.
#' 
#' From this structure, an analysis is done to retrieve, the type of data, the
#' version, the precision (very often it is empty), the format used (something
#' in "SHP", "GPKG" or "SQL"), the projection (something in "WGS84G", "LAMB93",
#' "RGAF09UTM20", "UTM22RGFG95", "RGR92UTM40S", "RGM04UTM38S"), the scope (
#' something in "FRA", "FXX", "GLP", "MTQ", "GUF", "REU", "MYT"), the date (in
#' the format YYYY-MM-DD), the year (YYYY), the extension, and the name (the
#' name of the archive without the extension.)
#' 
#' This is also done when the format is the less frequent case, because in this
#' case, the projection is not given.
#'
#' @param x character vector
#' @param origin character, to choose the format. Only "ign" is available at the
#' moment.
#'
#' @return either a vector or a matrix depending on the length of x.
#' 
#' @export
#' 
#' @encoding UTF-8
#' @author Frédéric Grelot
#'
#' @examples
#' ign = c(
#'   "ADMIN-EXPRESS-COG_3-1__SHP_LAMB93_FRA_2022-04-15.7z",
#'   "ADMIN-EXPRESS-COG_3-1__SHP_LAMB93_FRA_2022-04-15.7z.001",
#'   "ADMIN-EXPRESS-COG_3-1__SHP_LAMB93_FRA_2022-04-15",
#'   "ADMIN-EXPRESS-COG_3-1__SHP_LAMB93_BAD_2022-04-15",
#'   "ADMIN-EXPRESS-COG_3-1__SHP_BAD_FRA_2022-04-15",
#'   "ADMIN-EXPRESS-COG_3-1__BAD_LAMB93_FRA_2022-04-15",
#'   "ADMIN-EXPRESS-COG_3-1__SHP_LAMB93_FRA",
#'   "ADMIN-EXPRESS-COG_3-1__SHP_LAMB93_FRA_2022-04-15_toto"
#' )
#' analyse_archive(ign)
#' analyse_archive(rep(ign, 2))
#' 
#' floodam = c(
#'   "admin-express_2022-04-15",
#'   "admin-express_2022-04-15.rds",
#'   "admin-express_commune_2022-04-15",
#'   "admin-express_D034_2022-04-15",
#'   "admin-express_commune_D034_2022-04-15",
#'   "admin-express_D034_commune_2022-04-15",
#'   "ADMIN-EXPRESS-COG_3-1__SHP_RGAF09UTM20_GLP_2022-04-15.7z"
#' )
#' analyse_archive(floodam, "floodam.data")

analyse_archive = function(x, origin = c("ign", "floodam.data")) {
    origin = match.arg(origin)
    
    if (length(x) > 1) return(t(sapply(x, analyse_archive, origin = origin)))

    date_pattern = "[0-9]{4}-[0-9]{2}-[0-9]{2}"

    if (origin == "ign") {
        scope_admissible = c(
            "FRA",
            "FXX",
            "GLP",
            "MTQ",
            "GUF",
            "REU",
            "MYT"
        )
        proj_admissible = c(
            "WGS84G",
            "LAMB93",
            "RGAF09UTM20",
            "UTM22RGFG95",
            "RGR92UTM40S",
            "RGM04UTM38S",
            ""
        )
        type_admissible = c(
            "SHP",
            "GPKG",
            "SQL"
        )

        result = rep(NA_character_, 10)
        names(result) = c(
            "data",
            "version",
            "precision",
            "type",
            "projection",
            "scope",
            "vintage",
            "year",
            "extension",
            "name"
        )
        result["extension"] = basename_ext(x)
        result["name"] = basename_core(x)
        temp = strsplit(result["name"], "_")[[1]]

        # Dealing with unfrequent scheme
        if (length(temp) == 8) {
            if (temp[7] %in% proj_admissible) {
                temp[5] = temp[7]
                temp = temp[-7]
            }
        }

        # Check that result is compatible with frequent scheme
        if (length(temp) != 7) {
            warning(
                "analyse_archive WARNING: ",
                sprintf(
                    "%s slots found in %s. ",
                    length(temp),
                    result["name"]
                ), 
                "This is not compatible with IGN's scheme. ",
                "NAs returned.",
                call. = FALSE
            )
        } else {
            result[1:7] = temp
            result["year"] = strsplit(result["vintage"], "-")[[1]][1]
            result["scope"] = strsplit(result["scope"], "-")[[1]][1]

            # Checking projection
            result["projection"] = gsub("WM", "WGS84G", result["projection"])
            result["projection"] = gsub("L93", "LAMB93", result["projection"])
            if (! result["projection"] %in% proj_admissible) {
                warning(
                    "analyse_archive WARNING: ",
                    sprintf(
                        "%s is not an official IGN projection.",
                        result["projection"]
                    ),
                    call. = FALSE
                )
            }

            # Checking scope
            if (! result["scope"] %in% scope_admissible) {
                warning(
                    "analyse_archive WARNING: ",
                    sprintf(
                        "%s is not an official IGN scope.",
                        result["scope"]
                    ),
                    call. = FALSE
                )
            }

            # Checking type
            if (! result["type"] %in% type_admissible) {
                warning(
                    "analyse_archive WARNING: ",
                    sprintf(
                        "%s is not an official IGN type.",
                        result["type"]
                    ),
                    call. = FALSE
                )
            }
        }
    }

    if (origin == "floodam.data") {
        result = rep(NA_character_, 6)
        names(result) = c(
            "data",
            "precision",
            "scope",
            "vintage",
            "extension",
            "name"
        )
        result["extension"] = basename_ext(x)
        result["name"] = basename_core(x)
        temp = strsplit(result["name"], "_")[[1]]

        if (length(temp) > 4 | length(temp) == 0) {
            warning(
                "analyse_archive WARNING: ",
                sprintf(
                    "%s slot(s) found in %s. ",
                    length(temp),
                    result["name"]
                ), 
                "This is not compatible with floodam.data's scheme. ",
                "NAs returned.",
                call. = FALSE
            )
        } else {
            test = grep(date_pattern, temp[length(temp)], value = TRUE)       
            if (length(test) > 0) {
                result["vintage"] = temp[length(temp)]
                temp = temp[- length(temp)]
            }
            if (!is.na(format_scope(temp[length(temp)]))) {
                result["scope"] = format_scope(temp[length(temp)])
                temp = temp[- length(temp)]
            }
            if (length(temp) > 2) {
                warning(
                    "analyse_archive WARNING: ",
                    sprintf("%s cannot be analysed. ", result["name"]),
                    "Should be data_precision_scope_vintage, ",
                    "with possible missings but not reordering. ",
                    "NAs returned.",
                    call. = FALSE
                )
                result[1:4] = NA_character_
            } else {
                result[1:2] = temp[1:2]
            }
        }
    }

    # Replacing empty string by NA
    result[result == ""] = NA

    return(result)
}