#' @title Create a sequence of months 
#'
#' @description 
#' Create an increasing ordered sequence of months
#' 
#' @param begin character, begining month. See details for format.
#' @param ending character, ending month. See details for format.
#'
#' @return  A character vector with the sequence of month either in format "\%m" or "\%Y-\%m" depending on
#' parameters.
#' 
#' @details
#' It creates a sequence of ordered months. The final format depends on the format of input parameters, that must
#' be of the same type. If both begin and ending can be transform as `integer` between 1 and 12, it will return
#' a sequence of month of formatted month '\%m'. If both begin and ending can be interpreted as something like
#' "\%Y-\%m", it will return a sequence of month exactly formatted as "%Ym-%d". `begin` and `ending` may
#' be written as "\%Y_\%m", "\%Y:\%m", or "\%Y/\%m", eventually with missing starting'0' for month.
#'
#' @export
#'
#' @examples
#' ## Only with month (%m)
#' create_month(4, 8)
#' create_month("04", "08") 
#' create_month("08", "04") 
#' 
#' ## With year and month (%Y-%m)
#' 
#' create_month("2010-03", "2014-07")
#' create_month("2014-07", "2010-03")
#' create_month("2010-3", "2014-07")
#' create_month("2010-3", "2014-7")
#' 
#' \dontrun{create_month(4, 13)}
#' \dontrun{create_month("04", "13")}
#' \dontrun{create_month("2010-13", "2014-07")}
#' \dontrun{create_month("01", "2014-07")}

create_month = function(begin, ending) {
    format0 = function(x, width) {gsub(" ", "0", format(x, width = width, justify = "right"))}
    if(missing(ending)) ending = begin
    begin = as.integer(strsplit(as.character(begin)[1], "[-/_:]")[[1]])
    ending = as.integer(strsplit(as.character(ending)[1], "[-/_:]")[[1]])

    if(length(begin) == 1 & length(ending) == 1) {
        if(begin < 1 | begin > 12) {
            stop("Bad format for begin parameter (should be '%Y-%m' or '%m')")
        }
        if(ending < 1 | ending > 12) {
            stop("Bad format for ending parameter (should be '%Y-%m' or '%m')")
        }
        return(sort(format0(seq(begin, ending), width = 2)))
    }
    if(length(begin) > 2 | length(ending) > 2) {
        stop("Bad format for parameters (should be '%Y-%m' or '%m')")
    }
    if((length(begin) == 1 & length(ending) > 1) | (length(begin) > 1 & length(ending) == 1)) {
        stop("Incompatible formats for parameters")
    }
    if(as.integer(begin[2]) < 1 | as.integer(begin[2]) > 12) {
        stop("Bad format for begin parameter (should be '%Y-%m' or '%m')")
    }
    if(as.integer(ending[2]) < 1 | as.integer(ending[2]) > 12) {
        stop("Bad format for ending parameter (should be '%Y-%m' or '%m')")
    }
    result = expand.grid(format0(seq(begin[1], ending[1]), width = 4), format0(seq("01", "12"), width = 2))
    result = sort(paste(result[[1]], result[[2]], sep = "-"))
    limit = sort(c(
        paste(begin[1], format0(begin[2], 2), sep = "-"), 
        paste(ending[1], format0(ending[2], 2), sep = "-"))
    )
    result[result >= limit[1] & result <= limit[2]]
}
