#' Download Géo SIRENE 
#'
#' Function used to downlaod Geolocalized SIRENE data-base.
#'
#' @param destination character, the address where dara are stocked.
#' @param origin character, either a keyword or the address from where data are downloaded. Best to keep it
#' at default value. See details.
#' @param name character, vector of acceptable names fo archive to be downloaded.
#' @param extension character, vector of acceptable types of archive to be downloaded.
#' @param version string, version of ADMIN EXPRESS to be downloaded.
#' @param date character, date of the archive to be downloaded.
#' @param month character, vector of month to be downloaded
#' @param version_insee string, INSEE version to be downloaded. Only used with \code{origin = "cquest"}.
#' @param verbose logical, should the function send some messages while running.
#'
#' @return nothing
#'
#' @section Details:
#' 
#' If \code{origin == "cquest"}, then all necessary variables are filled with those values:
#' \itemize{
#'      \item \bold{origin} is changed to \url{https://data.cquest.org/geo_sirene}. This url is adapted depending
#'              on version_insee.
#'      \item \bold{version_insee} If value is "v2019", then the version 2019 is dowloaded from 
#'              \url{https://data.cquest.org/geo_sirene}.
#'      \item \bold{name} If version is not set to null, then default value is changed to "StockEtablissement_utf8",
#'          which should be the most complete version at France level.
#'      \item \bold{extension} default value is changed to "csv.gz".
#'      \item \bold{month} is used to set something equivalent to date, but directly in origin.
#'      \item \bold{date} is set to null, because conflict may occur with month.
#' }
#' 
#' If not, everything shall be filled so that `download_archive()` can make a successful download.
#' 
#'  @export
#'
#' @examples
#' \dontrun{
#' origin = "https://data.cquest.org/geo_sirene"
#' destination = tempdir()
#' month = c("2018-11", "2019-05", "2020-03")
#' download.geo_sirene(origin, destination, month = month)
#' unlink(destination)
#' }

download.geo_sirene = function(
    destination,
    origin = "cquest", 
    name = NULL,
    extension = NULL,
    version = NULL,
    date = NULL,
    month = NULL,
    version_insee = "v2019",
    verbose = TRUE) {
    if (origin == "cquest") {
        origin = "https://data.cquest.org/geo_sirene"
        if (version_insee == "v2019") {
            origin = file.path(origin, version_insee)
            version_insee = "version_2019"
        } else {
            version_insee = "version_2017"
        }
        if (is.null(month)) {
            x = read_url(origin)
            month = max(unlist(regmatches(x, gregexpr("(?<=href=([\\\"']))\\d{4}-\\d{2}(?=\\/\\1)", x, perl = TRUE)))) 
        }
        month_origin = month
        if (version_insee == "version_2019") month_origin = gsub("(2018|2019)", "\\1/\\1", month, perl = TRUE)
        origin = file.path(origin, month_origin)
        destination = file.path(destination, version_insee, month)
        if (is.null(extension)) extension = "csv.gz"
        if (is.null(version) && is.null(name)) name = "StockEtablissement_utf8"
        date = NULL
    }

    to_do = data.frame(
        origin = origin,
        destination = destination,
        short = "download.geo_sirene",
        stringsAsFactors = FALSE
    )

    download_archive(to_do, extension = extension, name = name, date = date, version = version, verbose = verbose)
}