#' Download BAN
#'
#' Function used to downlaod BAN
#'
#' @param destination character, the address where data are stocked.
#' @param origin character, either a keyword or the address from where data are downloaded. See details.
#' @param name character, vector of acceptable names for archive to be downloaded.
#' @param extension character, vector of acceptable types of archive to be downloaded.
#' @param date character, date of the archive to be downloaded.
#' @param version string, version of BAN to be downloaded. See details.
#' @param department integer, or converted to integer. Departments that should be considered. See details.
#' @param region integer, or converted to integer. Regions that should be considered. See details.
#' @param verbose logical, should the function send some messages while running.
#'
#' @return nothing
#' 
#' @section Details:
#' 
#' If \code{origin == "data.gouv"}, then all necessary variables are filled with
#' those values:
#' \itemize{
#'      \item \bold{origin} is changed to
#'      "https://adresse.data.gouv.fr/data/ban/adresses"
#'      \item \bold{name} default value is changed to "adresses"
#'      \item \bold{extension} default value is changed to "csv.gz"
#'      \item \bold{date} default value is used to choose a given directory. If
#'      date is null, then the most recent version is taken (equivalent to
#'      "latest").
#'      \item \bold{version} must be something within "ban" or "bal". When "ban"
#'      is used the file with one position per address is used. When "bal" is
#'      used, the file with several position per address is used.
#'      \item \bold{department} is used. Should be admissible French
#'      departments (a formatting will be attempted with `format_scope()`).
#'      If NULL, region may be used. If both are NULL, all departments from
#'      floodam.data::department will be used.
#'      \item \bold{region} may be used if department is null. Should be
#'      admissible French regions (a formatting will be attempted with
#'      `format_scope()`). If region is given and department is NULL, all
#'      departments from given region will be used.
#' }
#' 
#' If not, everything shall be filled so that `download_archive()` can make
#' a successful download.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' destination = tempdir()
#' download_ban(destination)
#' download_ban(destination, department = 34)
#' download_ban(destination, department = 34, date = "2022-01-01")
#' download_ban(destination, region = 11, date = "2022-01-01")
#' download_ban(destination, department = 34, date = "2022-09-20", version = "bal")
#' 
#' unlink(destination)
#' }

download_ban = function(
    destination,
    origin = "data.gouv", 
    name = NULL,
    extension = NULL,
    date = NULL,
    version = NULL,
    department = NULL,
    region = NULL,
    verbose = TRUE) {

    if(origin == "data.gouv") {
        version = match.arg(version, c("ban", "bal"))
        origin = "https://adresse.data.gouv.fr/data/ban/adresses"

        name = if(is.null(name)) "adresses"
        extension = if (is.null(extension)) "csv.gz"
        date = if (is.null(date)) "latest" else date
        date_admissible = names(
            get_archive(
                origin,
                name = "[0-9]{4}-[0-9]{2}-[0-9]{2}",
                html = "data.gouv",
                extension = NULL
            )
        )
        date = if (format(date) == "latest") {
            format(max(date_admissible))
        } else {
            format(date_admissible[which.min(abs(date_admissible - as.Date(date)))])
        }

        if ("ban" %in% version) {
            origin = file.path(origin, date, "csv")
            destination = file.path(destination, "ban", date)
        }
        if ("bal" %in% version) {
            origin = file.path(origin, date, "csv-bal")
            destination = file.path(destination, "bal", date)

        }
    
        if (!is.null(department)) {
            version = format_scope(department, type = "insee", scope = "department")
        } else {
            if (!is.null(region)) {
                region = format_scope(region, type = "insee", scope = "region")
                version = floodam.data::department[
                    floodam.data::department[["region"]] %in% region,
                    "code"]
            } else {
                version = floodam.data::department[["code"]]
            }
        }
    }

    to_do = data.frame(
        origin = origin,
        destination = destination,
        short = "download_ban",
        stringsAsFactors = FALSE
    )

    download_archive(
        to_do,
        extension = extension,
        name = name,
        date = NULL,
        version = version,
        html = "data.gouv",
        verbose = verbose
    )
}
