#' Extract data from journal files
#'
#' This function uses the contents of the journal file to find where are stocked formatted data and extract them to
#' data.frame. This function gives also the possibility to save those data.frame directly.
#'
#' @param origin character, path to the directory where journal is stored
#' @param archive string character, pattern to find the journal. If set to NULL, any .log file will be searched.
#' @param destination character, optional path to the directory where results should be saved. If set to NULL,
#' results are not saved.
#'
#' @return A list of data.frame depending on information stored in the journal.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot
#' 
#' @examples
#' 
#' \dontrun{
#' origin = "floodam.data/dev/data"
#' archive = "geo_siret"
#' result = extract.stat.log(origin, archive)
#' }

extract.stat.log = function(origin, archive = NULL, destination = NULL) {

    archive = grep("log$", dir(origin, pattern = archive), value = TRUE)
    if (length(archive) == 0) return(NULL)
    if (length(archive) > 1) {
        archive = archive[1]
        warning(sprintf("More than one journal can be analyzed. Only '%s' is analyzed.", archive))
    }
    
    retrieve.stat_in_content = function(x, n = 10) {
        n = max(7, as.integer(n), na.rm = TRUE)
        header = readLines(x, n)
        content = grep("^\t[0-9]+[.]", header)
        
        if (suppressWarnings(max(content) == n)) return(retrieve.stat_in_content(x, 2 * n))

        content = header[content]
        analyse = regmatches(content, regexec("Stats on (.*): ([0-9]+)-([0-9]+)$", content, perl = TRUE))
        analyse = analyse[sapply(analyse, length) == 4]
        
        if (length(analyse) == 0) return(NULL)

        result = as.data.frame(matrix(unlist(analyse), ncol = 4, byrow = TRUE), stringsAsFactors = FALSE)[-1]
        colnames(result) = c("theme", "skip", "nrows")
        result[["theme"]] = gsub(" ", "_", result[["theme"]] )
        result[["skip"]] = as.integer(result[["skip"]]) - 1
        result[["nrows"]] = as.integer(result[["nrows"]]) - result[["skip"]] - 1
        return(result)
    }

    to_do = retrieve.stat_in_content(file.path(origin, archive))

    if (is.null(to_do)) return(NULL)

    result = structure(
        mapply(utils::read.table, 
            skip = to_do[["skip"]], 
            nrows  = to_do[["nrows"]],
            MoreArgs = list(file = file.path(origin, archive)),
            SIMPLIFY = FALSE), 
        names = to_do[["theme"]])
    
    if(!is.null(destination)) {
        archive = gsub(".log", "", archive)
        mapply(utils::write.csv2, 
            result, 
            file.path(destination, sprintf("%s-%s.csv", names(result), archive)),
            MoreArgs = list(row.names = FALSE))  
    }

    return(result)
}
