#' @title Extract agricultural information building (layer in BD TOPO)
#'
#' @description
#' The function 'extract_building_agricultural' extracts polygons corresponding
#' to agricultural buildings from an sf object that should come from
#' 'extract_building' use.
#'
#' @param origin character, path to the directory where archive are stored.
#' @param destination character, path to the directory where results should be
#' saved.
#' @param archive character, vector of archive to be adpated.
#' @param journal logical, should a journal file be saved.
#' @param map logical, should a map be plotted (for check purpose).
#' @param retrieve logical, should the result be returned.
#' @param verbose logical, should the function give some sumup informations.
#'
#' @return if retrieve = TRUE, data.frame of adapated data stored in archive.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot
 
extract_building_agricultural = function(
    origin,
    destination,
    archive,
    journal = TRUE,
    map = TRUE,
    retrieve = FALSE,
    verbose = TRUE
) {
    ###  Recursive call if archive is missing
    if (missing(archive)) {archive = list.files(origin, pattern = ".rds$")}
    if (length(archive) > 1)
        return(
            lapply(
                archive,
                extract_building_agricultural,
                origin = origin,
                destination = destination,
                journal = journal,
                map = map,
                retrieve = retrieve,
                verbose = verbose
            )
        )
    if (length(archive) == 0)
        return(NULL)

    archive = file.path(origin, archive)
    
    cat(sprintf("'agricultural building' extraction from '%s'...\n", basename(archive)))
    old_s2 = suppressMessages(sf::sf_use_s2(FALSE))
    on.exit(suppressMessages(sf::sf_use_s2(old_s2)))

    agricultural = gsub("building", "agricultural", basename(archive))
    
    if (journal == TRUE) {
        journal = file.path(destination, gsub(".rds", ".log", agricultural))
        init_log_info(
            journal,
            main = sprintf("%s in %s", basename(archive), origin),
            treatment = "extract_building_agricultural",
        )
    } else {
        journal = FALSE
    }

    # Reading data
    result = add_log_info(archive, readRDS,
        info  = list(
            "short" = "read building",
            "long" = "Reading 'building' rds"
        ),
        journal = journal, verbose = verbose,
        verification = expression(info_dim(result))
    )

    # Analyse of nature & usage
    result = add_log_info(
        result,
        function(x) {
            x[
                select_from_usage(x[["nature"]], what = c("Silo", "Serre", "Moulin \u00E0 vent")) |
                select_from_usage(x[["destination_principal"]], what = "Agricole") |
                select_from_usage(x[["destination_secondary"]], what = "Agricole", na = FALSE), 
            ]
        },
        info  = list(
            "short" = "analyse 'nature' & 'destination'",
            "long" = "Analysing 'nature', 'destination_principal' and 'destination_secondary'"
        ),
        journal = journal, verbose = verbose,
        verification = expression(info_dim(result))
    )

    # Adding information
    result[["area"]] = add_log_info(result, sf::st_area,
        info  = list(
            "short" = "add 'area'",
            "long" = "Adding 'area' information"
        ),
        journal = journal, verbose = verbose,
        verification = expression(length(result))
    )
    result[["level"]] = add_log_info(
        result[["height"]],
        function(x) {pmax(round(x) %/% 3, 1)},
        info  = list(
            "short" = "add 'level'",
            "long" = "Adding 'level' information"
        ),
        journal = journal, verbose = verbose,
        verification = expression(length(result))
    )
    result[["developed"]] = add_log_info(
        result,
        function(x) {x[["area"]] * x[["level"]]},
        info  = list(
            "short" = "add 'developed'",
            "long" = "Adding 'developed' information"
        ),
        journal = journal, verbose = verbose,
        verification = expression(length(result))
    )

    # Adding 'typology' information
    result[["typology"]] = add_log_info(result, classify_building_agricultural,
        info  = list(
            "short" = "add 'typology'",
            "long" = "Classify 'building agricultural'"
        ),
        journal = journal, verbose = verbose,
        verification = expression(sum(!is.na(result)))
    )


    # Plot map
    if (map == TRUE) {
        add_log_info(result, map_log_building_agricultural,
            info  = list(
                "short" = "plot check map",
                "long" = "Plotting map (check purpose)"
            ),
            journal = journal, verbose = verbose,
            path_map = file.path(destination, gsub(".rds", ".png", agricultural))
        )
    }

    # Saving agricultural layer
    add_log_info(result, saveRDS,
        info  = list(
            "short" = "save",
            "long" = "Saving 'agricultural' layer"
        ),
        journal = journal, verbose = verbose,
        verification = expression(object.size(x)),
        file = file.path(destination, agricultural)
    )

    if (retrieve == TRUE) return(invisible(result))
    return(invisible())

}

classify_building_agricultural = function(x) {
    result = rep("agricultural", nrow(x))
    result[x[["nature"]] %in% "Serre"] = "greenhouse"
    result[x[["nature"]] %in% "Silo"] = "silo"
    result[x[["nature"]] %in% "Moulin \u00E0 vent"] = "mill"
    return(result)
}

map_log_building_agricultural = function(x, path_map) {
    grDevices::png(path_map, width = 1000, height = 1000)
    on.exit(grDevices::dev.off())

    col_eaip = c(
        "agricultural" = "red",
        "greenhouse" = "orange",
        "mill" = "green",
        "silo" = "blue"
    )
    col_safe = sapply(
        col_eaip,
        function(x) {grDevices::rgb(grDevices::colorRamp(c("white", x))(.2) / 255)}
    )
    col = col_eaip[x[["typology"]]]
    col[!x[["eaip"]]] = col_safe[x[["typology"]][!x[["eaip"]]]]
    col[is.na(col)] = "black"
    border = col
    plot(
        x[0],
        col = col,
        border = col,
        cex.axis = 2,
        axes = TRUE,
        xaxt = "n",
        yaxt = "n"
    )
    graphics::axis(2, at = pretty(sf::st_bbox(x)[c(2, 4)], 5), cex.axis = 2)
    graphics::axis(4, at = pretty(sf::st_bbox(x)[c(2, 4)], 5), cex.axis = 2)
    graphics::axis(1, at = pretty(sf::st_bbox(x)[c(1, 3)], 5), cex.axis = 2)
    graphics::axis(3, at = pretty(sf::st_bbox(x)[c(1, 3)], 5), cex.axis = 2)
    graphics::grid()

    return(NULL)
}
