#' @title Extract dwelling information building (layer in BD TOPO)
#'
#' @description
#' The function 'extract_dwelling' extracts polygons corresponding to dwellings
#' from an sf object that should come from 'extract_building' use.
#'
#' @param origin character, path to the directory where archive are stored.
#' @param destination character, path to the directory where results should be
#' saved.
#' @param archive character, vector of archive to be adpated.
#' @param journal logical, should a journal file be saved.
#' @param map logical, should a map be plotted (for check purpose).
#' @param department character, vector of department that should be treated
#' @param retrieve logical, should the result be returned.
#' @param verbose logical, should the function give some sumup informations.
#'
#' @return if retrieve = TRUE, data.frame of adapated data stored in archive.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot
 
extract_dwelling = function(
    origin,
    destination,
    archive,
    journal = TRUE,
    map = TRUE,
    department = NULL,
    retrieve = FALSE,
    verbose = TRUE
) {
    ###  Recursive call if archive is missing
    if (missing(archive)) {archive = list.files(origin, pattern = ".rds$")}
    if (length(archive) > 1)
        return(
            invisible(
                lapply(
                    archive,
                    extract_dwelling,
                    origin = origin,
                    destination = destination,
                    journal = journal,
                    map = map,
                    department = department,
                    retrieve = retrieve,
                    verbose = verbose
                )
            )
        )
    if (length(archive) == 0)
        return(NULL)

    archive = file.path(origin, archive)
    
    cat(sprintf("'dwelling' extraction from '%s'...\n", basename(archive)))
    old_s2 = suppressMessages(sf::sf_use_s2(FALSE))
    on.exit(suppressMessages(sf::sf_use_s2(old_s2)))

    dwelling = gsub("building", "dwelling", basename(archive))
    dir.create(destination, showWarnings = FALSE, recursive = TRUE)
    
    if (journal == TRUE) {
        journal = file.path(destination, gsub(".rds", ".log", dwelling))
        init_log_info(
            journal,
            main = sprintf("%s in %s", basename(archive), origin),
            treatment = "extract_dwelling",
        )
    } else {
        journal = FALSE
    }

    # Reading data
    result = add_log_info(archive, readRDS,
        info  = list(
            "short" = "read building",
            "long" = "Reading 'building' rds"
        ),
        journal = journal, verbose = verbose,
        verification = expression(info_dim(result))
    )

    # select_dwelling
    result = add_log_info(
        result,
        select_dwelling,
        info  = list(
            "short" = "select 'destination'",
            "long" = "select dwelling from 'destination_principal', 'destination_secondary', and 'dwelling'"
        ),
        journal = journal, verbose = verbose,
        verification = expression(info_dim(result))
    )

    # # Exclusion of leger
    # result = add_log_info(
    #     result,
    #     function(x) {
    #         x[x[["building_light"]] == FALSE, ]
    #     },
    #     info  = list(
    #         "short" = "exclude 'building_light'",
    #         "long" = "Exclusion of 'light building'"
    #     ),
    #     journal = journal, verbose = verbose,
    #     verification = expression(info_dim(result))
    # )

    # # Removing non-dwellings
    # result = add_log_info(
    #     result,
    #     function(x) {
    #         x[!is.na(x[["dwelling"]]), ]
    #     },
    #     info  = list(
    #         "short" = "remove 'dwelling' NA",
    #         "long" = "Removing variable with NA 'dwelling'"
    #     ),
    #     journal = journal, verbose = verbose,
    #     verification = expression(info_dim(result))
    # )

    # Adding information
    result[["terrace"]] = add_log_info(
        result,
        function(x) {
            sapply(
                suppressMessages(sf::st_intersects(x, x)),
                function(xx) {length(xx) > 1}
            )
        },
        info  = list(
            "short" = "add 'terrace'",
            "long" = "Adding 'terrace' information"
        ),
        journal = journal, verbose = verbose,
        verification = expression(length(result))
    )
    result[["area"]] = add_log_info(result, sf::st_area,
        info  = list(
            "short" = "add 'area'",
            "long" = "Adding 'area' information"
        ),
        journal = journal, verbose = verbose,
        verification = expression(length(result))
    )
    result[["level"]] = add_log_info(
        result[["height"]],
        function(x) {pmax(round(x) %/% 3, 1)},
        info  = list(
            "short" = "add 'level'",
            "long" = "Adding 'level' information"
        ),
        journal = journal, verbose = verbose,
        verification = expression(length(result))
    )
    result[["developed"]] = add_log_info(
        result,
        function(x) {x[["area"]] * x[["level"]]},
        info  = list(
            "short" = "add 'developed'",
            "long" = "Adding 'developed' information"
        ),
        journal = journal, verbose = verbose,
        verification = expression(length(result))
    )

    # Adding 'typology' information
    result[["typology"]] = add_log_info(result, classify_dwelling,
        info  = list(
            "short" = "add 'typology'",
            "long" = "Classify 'dwelling'"
        ),
        journal = journal, verbose = verbose,
        verification = expression(sum(!is.na(result)))
    )

    # Plot map
    if (map == TRUE) {

        add_log_info(result, map_log_dwelling,
            info  = list(
                "short" = "plot 'classification' map",
                "long" = "Plotting 'typology' map (check purpose)"
            ),
            journal = journal, verbose = verbose,
            path_map = file.path(destination, gsub(".rds", ".png", dwelling)),
            department = department
        )
        add_log_info(result, map_log_dwelling_nc,
            info  = list(
                "short" = "plot 'non classified' map",
                "long" = "Plotting 'non classified' map (check purpose)"
            ),
            journal = journal, verbose = verbose,
            path_map = file.path(destination, gsub(".rds", "-nc.png", dwelling)),
            department = department
        )
    }

    # Saving dwelling layer
    add_log_info(result, saveRDS,
        info  = list(
            "short" = "save",
            "long" = "Saving 'dwelling' layer"
        ),
        journal = journal, verbose = verbose,
        verification = expression(object.size(x)),
        file = file.path(destination, dwelling)
    )

    if (retrieve == TRUE) return(invisible(result))
    return(invisible())

}

classify_dwelling = function(x) {
    result = NA_character_
    result[!is.na(x[["dwelling"]]) & x[["dwelling"]] > 1] = "MFH"
    result[!is.na(x[["dwelling"]]) & x[["dwelling"]] > 10] = "AB"
    result[!is.na(x[["dwelling"]]) & x[["dwelling"]] == 1 & !x[["terrace"]]] = "SFH"
    result[!is.na(x[["dwelling"]]) & x[["dwelling"]] == 1 & x[["terrace"]]] = "TH"
    return(result)
}

# pick = (!is.na(dataset[["destination_principal"]]) & 
#     dataset[["destination_principal"]] == category) | 
#     (!is.na(dataset[["destination_secondary"]]) & 
#     dataset[["destination_secondary"]] == category) | 
#     (!is.na(dataset[["dwelling"]]) & dataset[["dwelling"]] > 0)

select_dwelling = function(x) {
    x[
        select_from_usage(x[["destination_principal"]], what = "R\u00E9sidentiel") |
        select_from_usage(x[["destination_secondary"]], na = FALSE, what = "R\u00E9sidentiel") |
        (!is.na(x[["dwelling"]]) & x[["dwelling"]] > 0)
        , 
    ]
}

select_from_usage = function(
    x,
    na = TRUE,
    what = c("R\u00E9sidentiel", "Indiff\u00E9renci\u00E9")
) {
    if (na == TRUE) {
        return(is.na(x) | x %in% what)
    } else {
        return(x %in% what)
    }
}

map_log_dwelling = function(x, path_map, department = NULL) {
    if (!is.null(department)) {
        department = file_version(
            department,
            "admin-express-cog_department.*rds"
        )
    }
    if (length(department) == 1) {
        department = readRDS(department)
    } else {
        department = NULL
    }

    grDevices::png(path_map, width = 1000, height = 1000)
    # grDevices::cairo_pdf(path_map, width = 7, height = 7)
    on.exit(grDevices::dev.off())

    x = x[order(x[["typology"]], decreasing = TRUE), ]
    col_typology = c(
        "AB" = "red",
        "MFH" = "orange",
        "TH" = "blue",
        "SFH" = "blue"
    )
    col = col_typology[x[["typology"]]]
    border = col
    plot(
        x[0],
        col = col,
        border = col,
        cex.axis = 2,
        axes = TRUE,
        xaxt = "n",
        yaxt = "n",
        reset = FALSE
    )
    if (!is.null(department))  {
        dep = unique(x[["department"]])
        col = ifelse(
            department[["department"]] == dep,
            scales::alpha("yellow", .05), scales::alpha("blue", .05)
        )
        plot(department[0], col = col, border = "black", add = TRUE)
    }
    graphics::axis(2, at = pretty(sf::st_bbox(x)[c(2, 4)], 5), cex.axis = 2)
    graphics::axis(4, at = pretty(sf::st_bbox(x)[c(2, 4)], 5), cex.axis = 2)
    graphics::axis(1, at = pretty(sf::st_bbox(x)[c(1, 3)], 5), cex.axis = 2)
    graphics::axis(3, at = pretty(sf::st_bbox(x)[c(1, 3)], 5), cex.axis = 2)
    graphics::grid()

    graphics::legend(
        x = "bottomright",
        inset = 1/100,
        legend = c("AB", "MFH", "SHF-TF"),
        fill = col_typology[c("AB", "MFH", "SFH")],
        cex = 2
    )
    return(NULL)
}

map_log_dwelling_nc = function(x, path_map, department = NULL) {
    if (!is.null(department)) {
        department = file_version(
            department,
            "admin-express-cog_department.*rds"
        )
    }
    if (length(department) == 1) {
        department = readRDS(department)
    } else {
        department = NULL
    }

    grDevices::png(path_map, width = 1000, height = 1000)
    on.exit(grDevices::dev.off())

    x = x[order(x[["typology"]]), ]
    col = c(
        "AB" = "gray",
        "MFH" = "gray",
        "TH" = "gray",
        "SFH" = "gray"
    )
    col = col[x[["typology"]]]
    col[is.na(col)] = "red"
    border = col
    plot(
        x[0],
        col = col,
        border = col,
        cex.axis = 2,
        axes = TRUE,
        xaxt = "n",
        yaxt = "n",
        reset = FALSE
    )
    if (!is.null(department))  {
        dep = unique(x[["department"]])
        col = ifelse(
            department[["department"]] == dep,
            scales::alpha("yellow", .05), scales::alpha("blue", .05)
        )
        plot(department[0], col = col, border = "black", add = TRUE)
    }
    graphics::axis(2, at = pretty(sf::st_bbox(x)[c(2, 4)], 5), cex.axis = 2)
    graphics::axis(4, at = pretty(sf::st_bbox(x)[c(2, 4)], 5), cex.axis = 2)
    graphics::axis(1, at = pretty(sf::st_bbox(x)[c(1, 3)], 5), cex.axis = 2)
    graphics::axis(3, at = pretty(sf::st_bbox(x)[c(1, 3)], 5), cex.axis = 2)
    graphics::grid()

    graphics::legend(
        x = "bottomright",
        inset = 1/100,
        legend = c("Non classified", "Classified"),
        fill = c("red", "gray"),
        cex = 2
    )

    return(NULL)
}
