#' @title Format name of archives to be consistent and predictible
#'
#' @description
#' `format_archive()` creates formatted names for archives given some
#' informations on the data contained in archives and the type of format aimed
#' at.
#'
#' It can be as the reciprocal function of
#' \code{\link[=analyse_archive]{analyse_archive()}}.
#'
#' @param x character, a named character.
#' @param extension character, if given, it will overrule what is in x.
#' @param origin character, default value to "floodam.data". See details.
#'
#' @return A vector of formatted names.
#' @seealso [analyse_archive()] 
#' 
#' @details
#' 
#' `x` should be a named character where informations used for formatting
#' the result ar stored. If some informations are missing, basically NA will be
#' used, and those NA will be post-treated depending on origin.
#' 
#' If origin is "floodam.data", `format_archive()` returns archive names
#' formatted with this general scheme: data_precision_scope_date. Missing
#' informations in `x` produce an adaptation of the scheme. For instance,
#' if nothing is given for "scope" in x or if x\["scope"\] is "NA", the resulting
#' scheme will be data_precision_date.
#' 
#' If origin is "ign", `format_archive()` returns names formatted with
#' this scheme: data-type_version_precision_format_projection_scope_date.
#' Missing informations in `x` are replaced by "".
#' 
#' If origin is "version", `format_archive()` returns names formatted with
#' this scheme: version_precision_format_projection_scope. Missing informations
#' in `x` are replaced by ".*". No extension information is used.
#' 
#' If extension is given, the extension is added to be name produced. I no
#' extension is given, x\["extension"\] will be used if not NA.
#' 
#' @export
#' 
#' @encoding UTF-8
#' @author Frédéric Grelot
#'
#' @examples
#' archive = "ADMIN-EXPRESS-COG_3-1__SHP_RGAF09UTM20_GLP_2022-04-15.7z"
#' x = analyse_archive(archive)
#'
#' format_archive(x, origin = "ign")
#' archive == format_archive(x, origin = "ign")
#' 
#' format_archive(x, extension = "rds", origin = "ign")
#' format_archive(x, extension = "rds")
#' x["precision"] = "commune"
#' x["scope"] = "D972"
#' format_archive(x, extension = "rds")
#'
#' format_archive(origin = "version")

format_archive = function(
    x,
    extension,
    origin = c("floodam.data", "ign", "version")
) {
    origin = match.arg(origin)

    result = NULL
    if (origin == "ign") {
        result = sprintf(
            "%s_%s_%s_%s_%s_%s_%s",
            toupper(x["data"]),
            x["version"],
            x["precision"],
            toupper(x["type"]),
            toupper(x["projection"]),
            format_scope(x["scope"], check = FALSE),
            x["vintage"]
        )
        result = gsub("NA", "", result)
    }
    if (origin == "floodam.data") {
        result = sprintf(
            "%s_%s_%s_%s",
            tolower(x["data"]),
            tolower(x["precision"]),
            format_scope(x["scope"]),
            x["vintage"]
        )
        result = gsub("_NA|NA_", "", result)
        if (result == "NA") result = ""
    }
    if (origin == "version") {
        if (missing(x)) x = character()
        result = sprintf(
            "%s_%s_%s_%s_%s",
            x["version"],
            x["precision"],
            x["type"],
            x["projection"],
            format_scope(x["scope"], check = FALSE)
        )
        result = gsub("NA", ".*", result)
        extension = NA
    }

    if (missing(extension)) extension = x["extension"]
    if (!is.na(extension)) result = sprintf("%s.%s", result, extension)

    return(result)
}

version_archive = function(
    version = ".*",
    precision = ".*",
    type = ".*",
    projection = ".*",
    scope = ".*"
) {
    make_default = function(x) {
        if (is.null(x)) x = ".*"
        if (length(x) == 0) x = ".*"
        return(x)
    }
    paste(
        make_default(version),
        make_default(precision),
        make_default(type),
        make_default(projection),
        make_default(scope),
        sep = "_"
    )
}
