#' @title Generate summary report of analysis
#' 
#' @description
#' The function generates a report from a template. For the moment it only works
#' for dwelling extracted from BD Topo.
#'
#' @param origin character, path to the directory where archive are stored.
#' @param destination character, path to the directory where output should be 
#' saved.
#' @param archive either a list with the information to display in the report or
#' a vector of rds filenames containing compressed lists to be processed.
#' @param quiet logical, should `rmarkdown::render()` be quiet or not.
#' @param template character, either a keyword or the file name for the
#' template to be used.
#' @param complete logical, should the report be complete or synthetic.
#' @param purge logical, should existing template in destination be removed and,
#' so, a copy from original template in library be copied.
#' @param verbose logical, should the function send some messages while running.
#'
#' @import kableExtra
#'
#' @export
#'
#' @encoding UTF-8
#' @author David Nortes Martinez

generate_report = function(
    origin,
    destination = origin,
    archive,
    quiet = TRUE,
    template = "dwelling",
    complete = FALSE,
    purge = FALSE,
    verbose = TRUE
){
    # Recursive call if archive is missing
    if (missing(archive)) {
        archive = list.files(origin, pattern = "analysed.rds$")
    }
    if (length(archive) > 1 & !(is.list(archive))) {
        # Some message TO BE CHANGED WITH LOG!!!
        if (verbose == TRUE) {
            msg = sprintf(
                "generate_report applied to %s (%s archives in progress)...",
                origin,
                length(archive)
            )
            message(msg)
        }
        return(
            invisible(
                lapply(
                    archive,
                    generate_report,
                    origin = origin,
                    destination = destination,
                    quiet = quiet,
                    template = template,
                    purge = purge,
                    verbose = verbose
                )
            )
        )
    }
    if (length(archive) == 0)
        return(NULL)

    # assigning value to tab_graphs depending on whether archive is a string or a list
    if (is.list(archive)) {
        tab_graphs = archive
        archive = "unmaned archive"
    } else {
        tab_graphs = readRDS(file.path(origin, archive))
    }
    if (verbose == TRUE) {
        message(sprintf("\t- %s:", basename(archive)))
    }

    # defining env (errors in some computers otherwise)
    e = new.env()
    on.exit(rm(e))
    assign("tab", tab_graphs, e)

    template = switch(
        template,
        "dwelling" = "template_bd-topo_dwelling.Rmd",
        template
    )

    if (complete == TRUE) {
        template = gsub(".Rmd", "_complete.Rmd", template)
    }

    if (purge == TRUE) {
        unlink(file.path(destination, template))
    }
    if (!file.exists(file.path(destination, template))) {
        template_path = try(
            system.file(
                file.path("rmd", template),
                package = 'floodam.data',
                mustWork = TRUE
            ),
            silent = TRUE
        )
        if (methods::is(template_path, "try-error")) {
            message("\t\t- No report rendered: No template found.")
            return(NULL)
        }

        file.copy(template_path, destination)
    }
    template = file.path(destination, template)

    # Creating report
    result = rmarkdown::render(
        template,
        envir = e, 
        output_file = tab_graphs[["report"]][["report_name"]],
        quiet = quiet
    )

    if (verbose == TRUE) {
        message(sprintf("\t\t- report rendered in %s", result))
    }
    return(invisible(NULL))
}
