#' @title Find archive within an html-type vector
#'
#' @description
#' Find all archive of given type within a vector of html text.
#'
#' @param origin character, the url address where to find archives.
#' @param extension character, vector of acceptable types of archives to be
#' downloaded.
#' @param name character, vector of acceptable names fo archives to be
#' downloaded. See details
#' @param date character, something like a date that should be used as a filter.
#' See details
#' @param version character, something like a version that should be used as a
#' filter.
#' @param directory logical, should directories be found instead of archives.
#' See details.
#' @param html character, indicates if the page is "simple" or is coming from
#' "data.gouv". This has implications on how to look for links.
#'
#' @return A character vector of all archives or directory found in origin
#' matching with given constraints.
#' 
#' @details
#' 
#' First, a regex search is made to find in x names enclosed in href="name" or
#' href='name'.
#' 
#' `extension` may contain different possibilities. It will be matches at
#' the end of archives' names. This may lead to an empty character as result.
#' 
#' `name` may contain different possibilities. It will be matched at the
#' beginning of archives' names. This may lead to an empty character as result.
#' 
#' `date` may contain either "last", and so anything that can be considered
#' as a date in archives' names ("\%Y", "\%Y-\%m", "\%Y-\%m-\%d") is matched
#' against and the max is taken. If nothing matches, all archives' names are
#' kept. code{date} may also contain anything admissible for code{create_date}.
#' If so, anything that can be considered as a date in archives' names ("\%Y",
#' "\%Y-\%m", "\%Y-\%m-\%d") is matched against, and only archives' names whose
#' date pertain to \code{create_date(date)} are kept, possibly nothing.
#' 
#' `version` may contain different possibilities. Il will be matched
#' anywhere in archives' names. This may lead an empty character as result.
#' 
#' If `directory` is set to TRUE, `extension` is not used. Instead,
#' links finishing by "\" are looked after.
#'
#' @export
#'
#' @examples
#' 
#' \dontrun{
#' # RPG archive for year 2010 in data.cquest.org
#' origin = "https://data.cquest.org/registre_parcellaire_graphique/2010"
#' file_list = get_archive(origin)
#' get_archive(origin)
#' get_archive(origin, version = "34")
#' get_archive(origin, version = 30:35)
#' 
#' # All RPG archives for any year for region "Occitanie" in data.cquest.org
#' origin = get_archive(
#'  "https://data.cquest.org/registre_parcellaire_graphique",
#'  directory = TRUE
#' )
#' get_archive(origin, version = "R76")
#' 
#' # "geo_siret" archives in data.cquest.org
#' origin = "https://data.cquest.org/geo_sirene/v2019/last/dep"
#' get_archive(origin, "gz", c("geo_siret_34", "geo_siret_83"))
#' get_archive(origin, "gz", c("geo_siret"), version = c("34", "83"))
#' 
#' # "ADMIN EXPRESS" archives in ign
#' origin = "https://geoservices.ign.fr/adminexpress"
#' get_archive(origin, "7z", "ADMIN-EXPRESS-COG", date = "last")
#' get_archive(origin, "7z", "ADMIN-EXPRESS-COG", version = "FRA", date = "last")
#' get_archive(origin, "7z", "ADMIN-EXPRESS", date = 2021:2022)
#' }

get_archive = function(
    origin,
    extension = c("zip", "7z"),
    name = NULL,
    date = NULL,
    version = NULL,
    directory = FALSE,
    html = c("simple", "data.gouv")
) {
    html = match.arg(html)

    if (length(origin) > 1) {
        result = lapply(
            origin,
            get_archive,
            extension = extension,
            name = name,
            date = date,
            version = version,
            directory = directory
        )
        names(result) = NULL
        result = result[-duplicated(result)]
        return(unlist(result))
    }

    x = read_url(origin)
    if ("try-error" %in% class(x)) {
        return(invisible())
    }

    if (directory == TRUE) {extension = "/"}
    if (html == "data.gouv") {
        result = unlist(strsplit(x, '"'))
        if (!is.null(extension)) {
            result = unique(
                grep(
                    complete_pattern(
                        extension,
                        extension = !directory,
                        ending = TRUE
                    ),
                    result,
                    value = TRUE)
            )
        }
        if (!is.null(name)) {
            result = unique(
                grep(
                    complete_pattern(sprintf("%s[_-]*", name), begin = TRUE),
                    result,
                    value = TRUE
                )
            )
        }
    } else {
        pattern = "(?<=href=([\"'])).*?(?=\\1)"
        result = grep(
            complete_pattern(extension, extension = !directory, ending = TRUE),
            unlist(regmatches(x, gregexpr(pattern, x, perl = TRUE))),
            value = TRUE
        )
        result = grep(
            "^(?![.]{1,2}$)",
            gsub("/$", "", result),
            perl = TRUE,
            value = TRUE
        )
    }

    selection = grepl("^http", result)
    result[!selection] = file.path(origin, result[!selection])
    result = stats::setNames(result, basename(result))

    if (!is.null(name)) {
        result = result[
            grep(
                complete_pattern(sprintf("%s[_-]*", name), begin = TRUE),
                names(result)
            )
        ]
    }
    if (!is.null(version)) {
        result = result[
            grep(
                complete_pattern(version),
                names(result)
            )
        ]
    }
    if (!is.null(date)) {
        date_result = regmatches(
            names(result),
            gregexpr(
                "(\\d{4}-\\d{2}-\\d{2}|\\d{4}-\\d{2}|\\d{4})",
                names(result),
                perl = TRUE
            )
        )
        date_result[sapply(date_result, length) == 1] =
            sapply(
                lapply(
                    date_result[sapply(date_result, length) == 1],
                    create_date
                ),
                "[",
                1
            )
        date_result[sapply(date_result, length) == 0] = "00000_nothing"
        date_result[sapply(date_result, length) > 1] = sapply(
            date_result[sapply(date_result, length) > 1],
            max
        )
        date_result = unlist(date_result)
        if (identical(date, "last")) {
            result = result[date_result == max(date_result, na.rm = TRUE)]
        } else {
            date = create_date(date)
            date_result[]
            result = result[date_result >= date[1] & date_result <= date[2]]
        }
    }
    return(result)
}
