#' @title Control if an object x is inside an object y
#'
#' @description
#' 'in_layer' checks if geometries in 'x' are, even partially, inside any
#' geometry in 'y'.
#' 
#' @param x object that is admissible for st_intersects.
#' @param y object that is admissible for st_intersects.
#' @param treatment character indicating which treatment should be done. See
#' details.
#' @param name character, name that will be used for results. Default to
#' "in_layer".
#' @param divide logical, should lwgeom::st_subdivide be used on y to have fast
#' results.
#' @param s2 logical, should sf_use_s2 be used or not.
#'
#' @return
#' If treatment = "intersect", logical with same length as nrow(x)
#' indicating whether geometry in x are (event partially) in y.
#' In any other case, data.frame whose column corresponds to asked treaments and
#' rows to geometry in x.
#' 
#' @details
#' When `treatment` includes "interesect", a logical is produced that
#' indicates if geometries in x intersects any geometries in y.
#' When `treatment` includes "intersection", a numeric is produced that
#' indicates for each geometry in x the area that is intersected by all
#' geometries in y.
#' When `treatment` includes "proportion", a numeric is produced that
#' indicates for each geometry in x the proportion of area that is intersected
#' by all geometries in y.
#' 
#' The use of `divide` should give fastet results in real case where y may
#' have a lot of details in its polygon defintion. As lwgeom::subdivide is
#' performed within the function and takes some time, it is a better strategy to
#' use `treatment` equals to "all" when both intersect and intersection are
#' asked, because it is done only one time for both treatments.
#' 
#' To have more consistent results st_union is performed on y each time
#' `treatment` includes either "intersection" or "proportion".
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot
#' 
#' @examples
#' fun_p1 = function(p, n, x, y, s = 0){
#'   fun = function(p, x, y, s) {p * s + c(x, y)}
#'   if (s == 0) s = 1 else s = runif(n) * s
#'   result = mapply(fun, runif(n) * x, runif(n) * y, s, MoreArgs = list(p = p), SIMPLIFY = FALSE)
#'   sf::st_sfc(result)
#' }
#' fun_p2 = function(n) {
#'   theta = seq(0, 1, length.out = n) * 2 * pi
#'   theta[n] =  theta[1]
#'   rho = .8 + runif(n) * .2
#'   rho[n] = rho[1]
#'   result = mapply(
#'     function(theta, rho) {c(sin(theta), cos(theta)) * rho},
#'     theta,
#'     rho,
#'     SIMPLIFY = FALSE
#'   )
#'   sf::st_polygon(list(do.call(rbind, result)))
#' }
#'
#' set.seed(0)
#' p1 = fun_p2(100)
#' p2 = fun_p2(100)
#' x = fun_p1(p1 * .2, 100, 10, 10)
#' y = fun_p1(p1, 25, 10, 10, .5)
#'
#' in_layer(x, y)
#' in_layer(y, x)
#' in_layer(x, y, "intersect")
#' in_layer(x, y, "intersection")
#' in_layer(x, y, "proportion")
#' in_layer(x, y, "all", "y")
#' in_layer(x, y, "all", "y", divide = FALSE)
#' 
#' \dontrun{
#' n = 5
#' z = sf::st_sfc(replicate(n, sf::st_union(fun_p1(p2 * .15, 1000, 10, 10, 0))))
#' z = sf::st_make_valid(sf::st_difference(z))
#'
#' plot(z, col = scales::alpha(rainbow(n), .5), axes = TRUE)
#' plot(x, col = scales::alpha("blue", .5), add = TRUE)
#' plot(y, col = scales::alpha("red", .5), add = TRUE)
#' 
#' microbenchmark::microbenchmark(
#'   slow = in_layer(x, z, divide = FALSE),
#'   fast = in_layer(x, z, divide = TRUE),
#'   times = 10
#' )
#' microbenchmark::microbenchmark(
#'   slow = in_layer(x, z, "intersection", divide = FALSE),
#'   fast = in_layer(x, z, "intersection", divide = TRUE),
#'   times = 10
#' )
#' microbenchmark::microbenchmark(
#'   slow = in_layer(x, z, "all", divide = FALSE),
#'   fast = in_layer(x, z, "all", divide = TRUE),
#'   times = 10
#' )
#'}
 
in_layer = function(
    x,
    y,
    treatment = "intersect",
    name = "in_layer",
    divide = TRUE,
    s2 = FALSE
) {
    arguments = c("intersect", "intersection", "proportion")
    if ("all" %in% treatment) treatment = arguments
    treatment = match.arg(treatment, arguments, several.ok = TRUE)
    name = as.character(name)[1]

    s2 = !s2 == FALSE
    old_s2 = suppressMessages(sf::sf_use_s2(s2))
    on.exit(suppressMessages(sf::sf_use_s2(old_s2)))
    
    if (divide == TRUE) {
        y = sf::st_union(y)
        y = sf::st_cast(lwgeom::st_subdivide(y, 50))
    }

    if (all(sf::st_geometry_type(x) == "POINT")) {
        result = rep(FALSE, nrow(x))
        result[unlist(suppressMessages(sf::st_contains(y, x)))] = TRUE
        return(result)
    }

    if ("intersect" %in% treatment) {
        in_y = lengths(suppressMessages(sf::st_intersects(x, y))) > 0
        if (length(treatment) == 1) return(in_y)
    }

    if (divide == FALSE) {
        y = sf::st_union(y)
    }

    check = all(lengths(sf::st_intersects(y, y)) == 1)
    if (!check) {
        warning(
            "Some elements of y are self-intersecting, ",
            "results may not be consistent."
        )
    }

    name_a = sprintf("%s_area", name)
    name_p = sprintf("%s_prop", name)

    x = sf::st_as_sf(x)
    x[["id_in_layer"]] = seq(nrow(x))
    intersection = suppressWarnings(sf::st_intersection(x["id_in_layer"], y))
    intersection[[name_a]] = sf::st_area(intersection)
    intersection = sf::st_drop_geometry(intersection)
    area = stats::aggregate(
        intersection[name_a],
        intersection["id_in_layer"], sum)

    result = merge(sf::st_drop_geometry(x["id_in_layer"]), area, all.x = TRUE)
    result = result[order(x[["id_in_layer"]]), ]
    result[is.na(result[[name_a]]), name_a] = 0
    result[[name_p]] = result[[name_a]] / sf::st_area(x)
    result[[name]] = if ("intersect" %in% treatment) in_y else NA

    selection = c(name, name_a, name_p)
    selection = selection[arguments %in% treatment]

    return(result[selection])
}

in_layer_old = function(x, y) {
    if (all(sf::st_geometry_type(x) == "POINT")) {
        result = rep(FALSE, nrow(x))
        result[unlist(suppressMessages(sf::st_contains(y, x)))] = TRUE
    } else {
        result = sapply(suppressMessages(sf::st_intersects(x, y)), any)
    }
    return(result)
}