#' @name read_with_scheme
#' @title Read data with a scheme
#' 
#' @description
#' `read_with_scheme()` is a wrapper for more specific functions. It uses
#' `type` and `extension` to call the good function. If those
#' parameters are not specified it tries to guess them from the name of the
#' archive included in `x`.
#'
#' @param x character, path of the data to be read.
#' @param scheme data.frame, scheme to be applied.
#' @param projection something that will be used in the option "crs" of 
#' \code{sf::st_as_sf}.
#' @param layer character, indicates the layer to be read for data organized by
#' layers.
#' @param type character, indicates the type of data that should be read.
#' @param extension character, indicate the exrtension of the archive. If
#' missing will be guessed from x.
#'
#' @return data adapted, either a data.frame or a 'sf' objects.
#' 
#' @details 
#' 
#' If 'scheme' is NULL (default). \code{data.table::fread} is used with
#' colClasses set to "character".
#' 
#' If 'projection' is NA (default), no crs is given \code{sf::to st_as_sf}.
#'
#' @rdname read_with_scheme
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot
 
read_with_scheme = function(
    x,
    scheme = NULL,
    projection = NA,
    layer,
    type,
    extension = tools::file_ext(x)
) {
    if (missing(type)) {
        test = suppressWarnings(analyse_archive(basename(x))[["type"]])
        if (!is.na(test)) {
            type = test
        } else if (extension == "gz") {
            type = tools::file_ext(tools::file_path_sans_ext(x))
        } else {
            type = extension
        }
    }
    type = tolower(type)
    extension = tolower(extension)

    if (type == "csv") {
        return(read_csv_with_scheme(x, scheme, projection))
    }
    if (extension == "shp") {
        return(read_shp_with_scheme(x, scheme, projection, layer))
    }
    if (extension == "gpkg") {
        return(read_gpkg_with_scheme(x, scheme, projection, layer))
    }
    if (extension == "7z" & type == "gpkg") {
        return(read_gpkg_from_7z(x, scheme, projection, layer))
    }
    if (extension == "7z" & type == "shp") {
        return(read_shp_from_7z(x, scheme, projection, layer))
    }
    if (type == "zip") {
        x = sprintf("unzip -p %s", x)
        old_option = options(datatable.fread.input.cmd.message = FALSE)
        on.exit(
            options(
                datatable.fread.input.cmd.message =
                    old_option[["datatable.fread.input.cmd.message"]]
            )
        )
        return(read_csv_with_scheme(x, scheme, projection))
    }

    message(
        sprintf(
            "Could not a way to read data of type '%s' and extension '%s'.",
            type,
            extension
        ),
        "NULL is returned."
    )
    return(NULL)
}

#' @rdname read_with_scheme
#' 
#' @description
#' `read_csv_with_scheme()` relies on \code{data.table::fread} to read
#' tabular data, and to apply a scheme. The idea is to used the efficiency of
#' `fread()` to read more quickly data (and without too many specifications
#' on "sep" and "dec"), and to used some options to read only data that will be
#' kept with original format given in scheme. As almost, everyhting is done
#' after reading, the remaining part of applying a scheme is also one: final
#' format is applied, and if, some "lon" and "lat" is found, data is transformed
#' in sf object.
#'
#' @export

read_csv_with_scheme = function(
    x,
    scheme = NULL,
    projection = NA
) {
    if (!is.null(scheme)) {
        scheme = scheme[scheme[["keep"]], ]
        select = stats::setNames(
            scheme[["type_source"]],
            scheme[["name_origin"]]
        )
        col.names = scheme[["name"]]
        result = data.table::fread(
            x,
            select = select,
            col.names = col.names,
            data.table = FALSE
        )

        treatment = list(
            "factor" = as.factor,
            "date" = as.Date,
            "numeric" = as.numeric,
            "integer" = as.integer
        )
        for (type in names(treatment)) {
            selection = scheme[["name"]][scheme[["type"]] == type]
            result[selection] = lapply(result[selection], treatment[[type]])
        }
    } else {
        result = data.table::fread(
            x,
            data.table = FALSE,
            colClasses = "character"
        )
    }

    if (all(c("lon", "lat") %in% names(result))) {
        result = sf::st_as_sf(
            result,
            coords = c("lon", "lat"),
            crs = projection
        )
    }

    return(result)
}

#' @rdname read_with_scheme
#' 
#' @description
#' `read_shp_with_scheme()` relies on \code{sf::st_read} to read gpkg data.
#' If given, projection is changed. If given scheme is applyed through
#' `apply_scheme()`. It is supposed that the scheme given is organized with a
#' layer column.
#' 
#' @export

read_shp_with_scheme = function(
    x,
    scheme = NULL,
    projection = NA,
    layer
) {
    result = sf::st_read(x, quiet = TRUE)
    
    if (!is.na(projection)) {
        result = sf::st_transform(result, crs = projection)
    }
    if (!is.null(scheme)) {
        if (missing(layer)) layer = basename_core(x)
        result = apply_scheme(result, scheme, selection = list("layer" = layer))
    }

    return(result)
}

#' @rdname read_with_scheme
#' 
#' @description
#' `read_gpkg_with_scheme()` relies on \code{sf::st_read} to read gpkg data.
#' If given, projection is changed. If given scheme is applyed through
#' `apply_scheme()`. It is supposed that the scheme given is organized with a
#' layer column.
#' 
#' @export

read_gpkg_with_scheme = function(
    x,
    scheme = NULL,
    projection = NA,
    layer
) {
    result = sf::st_read(x, layer = layer, quiet = TRUE)

    if (!is.na(projection)) {
        result = sf::st_transform(result, crs = projection)
    }
    if (!is.null(scheme)) {
        result = apply_scheme(result, scheme, selection = list("layer" = layer))
    }

    return(result)
}

#' @rdname read_with_scheme
#' 
#' @description
#' \code{read_shp_from_7z} uses a system call to 7z to uncompress a
#' 7z archive. It looks specifically for files that correspond to given layer.
#' It then reads the uncompressed gpkg file with a call to
#' `read_shp_with_scheme()`.
#'
#' @export

read_shp_from_7z = function(
    x,
    scheme = NULL,
    projection = NA,
    layer
) {
    path_temp = tempdir()
    on.exit(unlink(path_temp))
    file_list = gsub(
        ".+ ", "",
        grep(
            sprintf("/%s.[a-z]+$", layer),
            system(sprintf("7z l %s 2>&1", x), intern = TRUE),
            value = TRUE
        )
    )
    sapply(
        sprintf("7z e -y %s -o%s %s", x, path_temp, file_list),
        system,
        ignore.stdout = TRUE,
        wait = TRUE
    )

    x = file.path(
        path_temp,
        grep("shp$", basename(file_list), value = TRUE)
    )
    return(read_shp_with_scheme(x, scheme, projection))
}

#' @rdname read_with_scheme
#' 
#' @description
#' \code{read_gkpg_from_7z} uses a system call to 7z to uncompress a 7z archive.
#' It looks specifically for files with extension gkpg, and presumes that there
#' is only one. It then reads the uncompressed gpkg file with a call to
#' `read_gpkg_with_scheme()`.
#'
#' @export

read_gpkg_from_7z = function(
    x,
    scheme = NULL,
    projection = NA,
    layer
) {
    path_temp = tempdir()
    on.exit(unlink(path_temp))
    gpkg = gsub(
        ".+ ", "",
        grep(
            "gpkg$",
            system(sprintf("7z l %s 2>&1", x), intern = TRUE),
            value = TRUE
        )
    )
    if (length(gpkg) > 1) {
        message(
            "Too many gpkg archives found. ",
            "Only the first one will be treated."
        )
        gpkg = gpkg[1]
    }
    if (length(gpkg) == 0) {
        message("No gpkg found, NULL is returned.")
        return(NULL)
    }

    system(
        sprintf("7z e -y %s -o%s %s", x, path_temp, gpkg),
        ignore.stdout = TRUE, wait = TRUE
    )

    x = file.path(path_temp, basename(gpkg))
    return(read_gpkg_with_scheme(x, scheme, projection, layer))
}
